@@ -15,13 +15,17 @@ prune.completion = require("./utils/completion/installed-deep.js")
 function prune (args, cb) {
   //check if is a valid package.json file
   var jsonFile = path.resolve(npm.dir, "..", "package.json" )
-  readJson(jsonFile, log.warn, function (er, data) {
+  readJson(jsonFile, log.warn, function (er) {
     if (er) return cb(er)
     next()
   })
 
   function next() {
-    readInstalled(npm.prefix, npm.config.get("depth"), function (er, data) {
+    var opt = {
+      depth: npm.config.get("depth"),
+      dev: !npm.config.get("production") || npm.config.get("dev")
+    }
+    readInstalled(npm.prefix, opt, function (er, data) {
       if (er) return cb(er)
       prune_(args, data, cb)
     })
