@@ -1,16 +1,16 @@
 
 module.exports = publish
 
-var npm = require("./npm.js")
+var url = require("url")
+  , npm = require("./npm.js")
   , log = require("npmlog")
-  , tar = require("./utils/tar.js")
   , path = require("path")
   , readJson = require("read-package-json")
-  , fs = require("graceful-fs")
   , lifecycle = require("./utils/lifecycle.js")
   , chain = require("slide").chain
   , Conf = require("npmconf").Conf
   , RegClient = require("npm-registry-client")
+  , mapToRegistry = require("./utils/map-to-registry.js")
 
 publish.usage = "npm publish <tarball>"
               + "\nnpm publish <folder>"
@@ -24,7 +24,10 @@ publish.completion = function (opts, cb) {
 }
 
 function publish (args, isRetry, cb) {
-  if (typeof cb !== "function") cb = isRetry, isRetry = false
+  if (typeof cb !== "function") {
+    cb = isRetry
+    isRetry = false
+  }
   if (args.length === 0) args = ["."]
   if (args.length !== 1) return cb(publish.usage)
 
@@ -49,15 +52,15 @@ function publish (args, isRetry, cb) {
 // That means that we can run publish/postpublish in the dir, rather than
 // in the cache dir.
 function cacheAddPublish (dir, didPre, isRetry, cb) {
-  npm.commands.cache.add(dir, function (er, data) {
+  npm.commands.cache.add(dir, null, null, false, function (er, data) {
     if (er) return cb(er)
     log.silly("publish", data)
     var cachedir = path.resolve( npm.cache
                                , data.name
                                , data.version
                                , "package" )
-    chain
-      ( [ !didPre && [lifecycle, data, "prepublish", cachedir]
+    chain([ !didPre &&
+          [lifecycle, data, "prepublish", cachedir]
         , [publish_, dir, data, isRetry, cachedir]
         , [lifecycle, data, "publish", didPre ? dir : cachedir]
         , [lifecycle, data, "postpublish", didPre ? dir : cachedir] ]
@@ -68,45 +71,61 @@ function cacheAddPublish (dir, didPre, isRetry, cb) {
 function publish_ (arg, data, isRetry, cachedir, cb) {
   if (!data) return cb(new Error("no package.json file found"))
 
-  // check for publishConfig hash
   var registry = npm.registry
+  var config = npm.config
+
+  // check for publishConfig hash
   if (data.publishConfig) {
-    var pubConf = new Conf(npm.config)
+    config = new Conf(npm.config)
+    config.save = npm.config.save.bind(npm.config)
 
     // don't modify the actual publishConfig object, in case we have
     // to set a login token or some other data.
-    pubConf.unshift(Object.keys(data.publishConfig).reduce(function (s, k) {
+    config.unshift(Object.keys(data.publishConfig).reduce(function (s, k) {
       s[k] = data.publishConfig[k]
       return s
     }, {}))
-    registry = new RegClient(pubConf)
+    registry = new RegClient(config)
   }
 
   data._npmVersion = npm.version
-  data._npmUser = { name: npm.config.get("username")
-                  , email: npm.config.get("email") }
 
   delete data.modules
-  if (data.private) return cb(new Error
-    ("This package has been marked as private\n"
-    +"Remove the 'private' field from the package.json to publish it."))
+  if (data.private) return cb(
+    new Error(
+      "This package has been marked as private\n" +
+      "Remove the 'private' field from the package.json to publish it."
+    )
+  )
 
-  var tarball = cachedir + ".tgz"
-  registry.publish(data, tarball, function (er) {
-    if (er && er.code === "EPUBLISHCONFLICT"
-        && npm.config.get("force") && !isRetry) {
-      log.warn("publish", "Forced publish over "+data._id)
-      return npm.commands.unpublish([data._id], function (er) {
-        // ignore errors.  Use the force.  Reach out with your feelings.
-        // but if it fails again, then report the first error.
-        publish([arg], er || true, cb)
-      })
-    }
-    // report the unpublish error if this was a retry and unpublish failed
-    if (er && isRetry && isRetry !== true) return cb(isRetry)
+  mapToRegistry(data.name, config, function (er, registryURI) {
     if (er) return cb(er)
-    console.log("+ " + data._id)
-    cb()
+
+    var tarball = cachedir + ".tgz"
+
+    // we just want the base registry URL in this case
+    var registryBase = url.resolve(registryURI, ".")
+    log.verbose("publish", "registryBase", registryBase)
+
+    var c = config.getCredentialsByURI(registryBase)
+    data._npmUser = {name: c.username, email: c.email}
+
+    registry.publish(registryBase, data, tarball, function (er) {
+      if (er && er.code === "EPUBLISHCONFLICT"
+          && npm.config.get("force") && !isRetry) {
+        log.warn("publish", "Forced publish over " + data._id)
+        return npm.commands.unpublish([data._id], function (er) {
+          // ignore errors.  Use the force.  Reach out with your feelings.
+          // but if it fails again, then report the first error.
+          publish([arg], er || true, cb)
+        })
+      }
+      // report the unpublish error if this was a retry and unpublish failed
+      if (er && isRetry && isRetry !== true) return cb(isRetry)
+      if (er) return cb(er)
+      console.log("+ " + data._id)
+      cb()
+    })
   })
 }
 
