@@ -9,6 +9,8 @@ var cbCalled = false
   , path = require("path")
   , wroteLogFile = false
   , exitCode = 0
+  , rollbacks = npm.rollbacks
+  , chain = require("slide").chain
 
 
 process.on("exit", function (code) {
@@ -22,13 +24,18 @@ process.on("exit", function (code) {
     }
 
     if (wroteLogFile) {
-      log.error("", [""
-                ,"Additional logging details can be found in:"
+      // just a line break
+      if (log.levels[log.level] <= log.levels.error) console.error("")
+
+      log.error("",
+                ["Please include the following file with any support request:"
                 ,"    " + path.resolve("npm-debug.log")
                 ].join("\n"))
       wroteLogFile = false
     }
-    log.error("not ok", "code", code)
+    if (code) {
+      log.error("code", code)
+    }
   }
 
   var doExit = npm.config.get("_exit")
@@ -50,8 +57,24 @@ function exit (code, noLog) {
   log.verbose("exit", [code, doExit])
   if (log.level === "silent") noLog = true
 
-  if (code && !noLog) writeLogFile(reallyExit)
-  else rm("npm-debug.log", function () { rm(npm.tmp, reallyExit) })
+  if (rollbacks.length) {
+    chain(rollbacks.map(function (f) {
+      return function (cb) {
+        npm.commands.unbuild([f], true, cb)
+      }
+    }), function (er) {
+      if (er) {
+        log.error("error rolling back", er)
+        if (!code) errorHandler(er)
+        else reallyExit(er)
+      } else {
+        rm("npm-debug.log", reallyExit)
+      }
+    })
+    rollbacks.length = 0
+  }
+  else if (code && !noLog) writeLogFile(reallyExit)
+  else reallyExit()
 
   function reallyExit() {
     // truncate once it's been written.
@@ -69,7 +92,6 @@ function exit (code, noLog) {
 
 
 function errorHandler (er) {
-  var printStack = false
   // console.error("errorHandler", er)
   if (!npm.config || !npm.config.loaded) {
     // logging won't work unless we pretend that it's ready
@@ -94,13 +116,55 @@ function errorHandler (er) {
   var m = er.code || er.message.match(/^(?:Error: )?(E[A-Z]+)/)
   if (m && !er.code) er.code = m
 
+  ; [ "type"
+    , "fstream_path"
+    , "fstream_unc_path"
+    , "fstream_type"
+    , "fstream_class"
+    , "fstream_finish_call"
+    , "fstream_linkpath"
+    , "stack"
+    , "fstream_stack"
+    , "statusCode"
+    , "pkgid"
+    ].forEach(function (k) {
+      var v = er[k]
+      if (!v) return
+      if (k === "fstream_stack") v = v.join("\n")
+      log.verbose(k, v)
+    })
+
+  log.verbose("cwd", process.cwd())
+
+  var os = require("os")
+  // log.error("System", os.type() + " " + os.release())
+  // log.error("command", process.argv.map(JSON.stringify).join(" "))
+  // log.error("node -v", process.version)
+  // log.error("npm -v", npm.version)
+  log.error("", os.type() + " " + os.release())
+  log.error("argv", process.argv.map(JSON.stringify).join(" "))
+  log.error("node", process.version)
+  log.error("npm ", "v" + npm.version)
+
+  ; [ "file"
+    , "path"
+    , "code"
+    , "errno"
+    , "syscall"
+    ].forEach(function (k) {
+      var v = er[k]
+      if (v) log.error(k, v)
+    })
+
+  // just a line break
+  if (log.levels[log.level] <= log.levels.error) console.error("")
+
   switch (er.code) {
   case "ECONNREFUSED":
     log.error("", er)
     log.error("", ["\nIf you are behind a proxy, please make sure that the"
               ,"'proxy' config is set properly.  See: 'npm help config'"
               ].join("\n"))
-    printStack = true
     break
 
   case "EACCES":
@@ -108,11 +172,9 @@ function errorHandler (er) {
     log.error("", er)
     log.error("", ["\nPlease try running this command again as root/Administrator."
               ].join("\n"))
-    printStack = true
     break
 
   case "ELIFECYCLE":
-    er.code = "ELIFECYCLE"
     log.error("", er.message)
     log.error("", ["","Failed at the "+er.pkgid+" "+er.stage+" script."
               ,"This is most likely a problem with the "+er.pkgname+" package,"
@@ -126,7 +188,6 @@ function errorHandler (er) {
     break
 
   case "ENOGIT":
-    er.code = "ENOGIT"
     log.error("", er.message)
     log.error("", ["","Failed using git."
               ,"This is most likely not a problem with npm itself."
@@ -135,7 +196,6 @@ function errorHandler (er) {
     break
 
   case "EJSONPARSE":
-    er.code = "EJSONPARSE"
     log.error("", er.message)
     log.error("", "File: "+er.file)
     log.error("", ["Failed to parse package.json data."
@@ -145,43 +205,35 @@ function errorHandler (er) {
               ].join("\n"), "JSON.parse")
     break
 
+  // TODO(isaacs)
+  // Add a special case here for E401 and E403 explaining auth issues?
+
   case "E404":
-    er.code = "E404"
     var msg = [er.message]
     if (er.pkgid && er.pkgid !== "-") {
       msg.push("", "'"+er.pkgid+"' is not in the npm registry."
-              ,"You should bug the author to publish it")
+              ,"You should bug the author to publish it (or use the name yourself!)")
       if (er.parent) {
         msg.push("It was specified as a dependency of '"+er.parent+"'")
       }
-      if (er.pkgid.match(/^node[\.\-]|[\.\-]js$/)) {
-        var s = er.pkgid.replace(/^node[\.\-]|[\.\-]js$/g, "")
-        if (s !== er.pkgid) {
-          s = s.replace(/[^a-z0-9]/g, ' ')
-          msg.push("\nMaybe try 'npm search " + s + "'")
-        }
-      }
       msg.push("\nNote that you can also install from a"
-              ,"tarball, folder, or http url, or git url.")
+              ,"tarball, folder, http url, or git url.")
     }
+    // There's no need to have 404 in the message as well.
+    msg[0] = msg[0].replace(/^404\s+/, "")
     log.error("404", msg.join("\n"))
     break
 
   case "EPUBLISHCONFLICT":
-    er.code = "EPUBLISHCONFLICT"
     log.error("publish fail", ["Cannot publish over existing version."
               ,"Update the 'version' field in package.json and try again."
               ,""
-              ,"If the previous version was published in error, see:"
-              ,"    npm help unpublish"
-              ,""
               ,"To automatically increment version numbers, see:"
               ,"    npm help version"
               ].join("\n"))
     break
 
   case "EISGIT":
-    er.code = "EISGIT"
     log.error("git", [er.message
               ,"    "+er.path
               ,"Refusing to remove it. Update manually,"
@@ -190,7 +242,6 @@ function errorHandler (er) {
     break
 
   case "ECYCLE":
-    er.code = "ECYCLE"
     log.error("cycle", [er.message
               ,"While installing: "+er.pkgid
               ,"Found a pathological dependency case that npm cannot solve."
@@ -199,7 +250,6 @@ function errorHandler (er) {
     break
 
   case "EBADPLATFORM":
-    er.code = "EBADPLATFORM"
     log.error("notsup", [er.message
               ,"Not compatible with your operating system or architecture: "+er.pkgid
               ,"Valid OS:    "+er.os.join(",")
@@ -268,51 +318,30 @@ function errorHandler (er) {
       break
     } // else passthrough
 
+  case "ENOSPC":
+    log.error("nospc", [er.message
+              ,"This is most likely not a problem with npm itself"
+              ,"and is related to insufficient space on your system."
+              ].join("\n"))
+    break
+
+  case "EROFS":
+    log.error("rofs", [er.message
+              ,"This is most likely not a problem with npm itself"
+              ,"and is related to the file system being read-only."
+              ,"\nOften virtualized file systems, or other file systems"
+              ,"that don't support symlinks, give this error."
+              ].join("\n"))
+    break
+
   default:
-    log.error("", er.stack || er.message || er)
-    log.error("", ["If you need help, you may report this *entire* log,"
-                  ,"including the npm and node versions, at:"
+    log.error("", er.message || er)
+    log.error("", ["", "If you need help, you may report this error at:"
                   ,"    <http://github.com/npm/npm/issues>"
                   ].join("\n"))
-    printStack = false
     break
   }
 
-  var os = require("os")
-  // just a line break
-  if (log.levels[log.level] <= log.levels.error) console.error("")
-  log.error("System", os.type() + " " + os.release())
-  log.error("command", process.argv
-            .map(JSON.stringify).join(" "))
-  log.error("cwd", process.cwd())
-  log.error("node -v", process.version)
-  log.error("npm -v", npm.version)
-
-  ; [ "file"
-    , "path"
-    , "type"
-    , "syscall"
-    , "fstream_path"
-    , "fstream_unc_path"
-    , "fstream_type"
-    , "fstream_class"
-    , "fstream_finish_call"
-    , "fstream_linkpath"
-    , "code"
-    , "errno"
-    , "stack"
-    , "fstream_stack"
-    ].forEach(function (k) {
-      var v = er[k]
-      if (k === "stack") {
-        if (!printStack) return
-        if (!v) v = er.message
-      }
-      if (!v) return
-      if (k === "fstream_stack") v = v.join("\n")
-      log.error(k, v)
-    })
-
   exit(typeof er.errno === "number" ? er.errno : 1)
 }
 
@@ -324,17 +353,16 @@ function writeLogFile (cb) {
 
   var fs = require("graceful-fs")
     , fstr = fs.createWriteStream("npm-debug.log")
-    , util = require("util")
     , os = require("os")
     , out = ""
 
   log.record.forEach(function (m) {
     var pref = [m.id, m.level]
     if (m.prefix) pref.push(m.prefix)
-    pref = pref.join(' ')
+    pref = pref.join(" ")
 
     m.message.trim().split(/\r?\n/).map(function (line) {
-      return (pref + ' ' + line).trim()
+      return (pref + " " + line).trim()
     }).forEach(function (line) {
       out += line + os.EOL
     })
